package com.muyu.product.domain.model;

import com.muyu.common.core.web.domain.BaseEntity;
import com.muyu.product.domain.ProjectInfo;
import com.muyu.product.domain.req.ProjectInfoEditReq;
import io.swagger.annotations.ApiModelProperty;
import lombok.*;
import lombok.experimental.SuperBuilder;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * @author DongZl
 * @description: 商品添加模型
 * @Date 2024-3-22 上午 10:50
 */
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
@EqualsAndHashCode(callSuper = true)
public class ProjectAddModel extends BaseEntity {

    private Long id;

    /** 商品名称 */
    @NotBlank(message = "商品名称不能为空")
    @ApiModelProperty(name = "name", value = "商品名称", required = true)
    private String name;

    /** 商品描述 */
    @ApiModelProperty(name = "introduction", value = "商品描述")
    private String introduction;

    /** 主类型 */
    @ApiModelProperty(name = "mianType", value = "主类型")
    private Long mianType;

    /** 父类型 */
    @ApiModelProperty(name = "parentType", value = "父类型")
    private Long parentType;

    /** 商品类型 */
    @NotNull(message = "商品类型不能为空")
    @ApiModelProperty(name = "type", value = "商品类型", required = true)
    private Long type;

    /** 商品主图 */
    @NotBlank(message = "商品主图不能为空")
    @ApiModelProperty(name = "image", value = "商品主图", required = true)
    private String image;

    /** 商品轮播图 */
    @ApiModelProperty(name = "carouselImages", value = "商品轮播图(多个图片用逗号分隔)")
    private String carouselImages;

    /** 商品状态(0-下架 1-上架) */
    @ApiModelProperty(name = "status", value = "商品状态(0-下架 1-上架)")
    private String status;

    /** 规格ID */
    @ApiModelProperty(name = "ruleId", value = "规格ID")
    private Long ruleId;

    /** 品牌ID */
    @ApiModelProperty(name = "brandId", value = "品牌ID")
    private Long brandId;

    /** SKU列表 */
    @ApiModelProperty(name = "skuList", value = "SKU列表")
    private List<ProductSkuModel> skuList;

    /** 销售价格 */
    @ApiModelProperty(name = "salePrice", value = "销售价格")
    private Double salePrice;

    /** 市场价格 */
    @ApiModelProperty(name = "marketPrice", value = "市场价格")
    private Double marketPrice;

    /** 库存数量 */
    @ApiModelProperty(name = "stock", value = "库存数量")
    private Integer stock;

    /** 排序号 */
    @ApiModelProperty(name = "sort", value = "排序号")
    private Integer sort;

    /** 是否推荐(0-否 1-是) */
    @ApiModelProperty(name = "isRecommend", value = "是否推荐(0-否 1-是)")
    private String isRecommend;

    /** 商品详情 */
    @ApiModelProperty(name = "detail", value = "商品详情")
    private String detail;

    /** 规格属性JSON */
    @ApiModelProperty(name = "specJson", value = "规格属性JSON")
    private String specJson;


    public static ProjectInfo editBuild(Long id, ProjectInfoEditReq projectInfoEditReq){
        return ProjectInfo.builder()
                .id(id)
                .name(projectInfoEditReq.getProjectAddModel().getName())
                .introduction(projectInfoEditReq.getProjectAddModel().getIntroduction())
                .mianType(projectInfoEditReq.getProjectAddModel().getMianType())
                .parentType(projectInfoEditReq.getProjectAddModel().getParentType())
                .type(projectInfoEditReq.getProjectAddModel().getType())
                .image(projectInfoEditReq.getProjectAddModel().getImage())
                .carouselImages(projectInfoEditReq.getProjectAddModel().getCarouselImages())
                .status(projectInfoEditReq.getProjectAddModel().getStatus())
                .ruleId(projectInfoEditReq.getProjectAddModel().getRuleId())
                .brandId(projectInfoEditReq.getProjectAddModel().getBrandId())
                .build();
    }
}
