package com.muyu.product.domain.req;

import com.muyu.product.domain.ProjectInfo;
import com.muyu.product.domain.ProjectSkuInfo;
import com.muyu.product.domain.model.ProjectAddModel;
import com.muyu.product.domain.model.ProjectModel;
import com.muyu.product.domain.model.ProjectSKUModel;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.AllArgsConstructor;
import lombok.experimental.SuperBuilder;
import io.swagger.annotations.*;
import com.muyu.common.core.web.domain.BaseEntity;

import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.List;

/**
 * 商品信息对象 project_info
 *
 * @author DongZeLiang
 * @date 2024-02-27
 */
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
@ApiModel(value = "ProjectInfoEditReq", description = "商品信息")
public class ProjectInfoEditReq extends BaseEntity {

    private static final long serialVersionUID = 1L;

    /** 商品名称 */
    @ApiModelProperty(name = "商品名称", value = "商品名称")
    private String name;

    /** 商品描述 */
    @ApiModelProperty(name = "商品描述", value = "商品描述")
    private String introduction;

    /** 主类型 */
    @ApiModelProperty(name = "主类型", value = "主类型")
    private Long mianType;

    /** 父类型 */
    @ApiModelProperty(name = "父类型", value = "父类型")
    private Long parentType;

    /** 商品类型 */
    @ApiModelProperty(name = "商品类型", value = "商品类型")
    private Long type;

    /** 商品图片 */
    @ApiModelProperty(name = "商品图片", value = "商品图片")
    private String image;

    /** 商品轮播图 */
    @ApiModelProperty(name = "商品轮播图", value = "商品轮播图")
    private String carouselImages;

    /** 商品状态 */
    @ApiModelProperty(name = "商品状态", value = "商品状态")
    private String status;

    /** 规格 */
    @ApiModelProperty(name = "规格", value = "规格")
    private Long ruleId;

    /** 品牌 */
    @ApiModelProperty(name = "品牌", value = "品牌")
    private Long brandId;


    @ApiModelProperty(value = "商品信息", required = true)
    @NotNull(message = "商品信息不能为空")
    private ProjectInfo projectInfo;

    @ApiModelProperty("SKU列表")
    private List<ProjectSkuInfo> skuList;

    // 添加非空校验注解
    @NotNull(message = "商品信息不能为空")
    public ProjectInfo getProjectInfo() {
        return projectInfo;
    }

    public List<ProjectSkuInfo> getSkuList() {
        return skuList != null ? skuList : new ArrayList<>();
    }

    public void setProjectInfo(ProjectInfo projectInfo) {
        this.projectInfo = projectInfo;
    }

    public void setSkuList(List<ProjectSkuInfo> skuList) {
        this.skuList = skuList;
    }

    private ProjectAddModel projectAddModel;
    private List<ProjectModel> attrValueList;
    private List<ProjectSKUModel> productSkuList;

}
