package com.muyu.product.service.impl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.function.Function;
import java.util.stream.Collectors;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.muyu.common.core.text.Convert;
import com.muyu.common.core.utils.ObjUtils;
import com.muyu.common.core.utils.StringUtils;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.product.domain.ProjectInfo;
import com.muyu.product.domain.RuleAttrInfo;
import com.muyu.product.domain.model.RuleAttrAddModel;
import com.muyu.product.domain.model.RuleInfoAddModel;
import com.muyu.product.domain.req.RuleAttrInfoReq;
import com.muyu.product.domain.req.RuleGroupUpdResp;
import com.muyu.product.domain.req.RuleInfoEditReq;
import com.muyu.product.domain.req.RuleInfoQueryReq;
import com.muyu.product.domain.resp.RuleInfoResp;
import com.muyu.product.mapper.ProjectInfoMapper;
import com.muyu.product.mapper.RuleAttrInfoMapper;
import com.muyu.product.service.RuleAttrInfoService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.math3.stat.descriptive.summary.Product;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.muyu.product.mapper.RuleInfoMapper;
import com.muyu.product.domain.RuleInfo;
import com.muyu.product.service.RuleInfoService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;

/**
 * 商品规格Service业务层处理
 *
 * @author DongZeLiang
 * @date 2024-02-27
 */
@Slf4j
@Service
public class RuleInfoServiceImpl extends ServiceImpl<RuleInfoMapper, RuleInfo>  implements RuleInfoService {

    @Autowired
    private RuleAttrInfoService ruleAttrInfoService;

    @Autowired
    private RuleAttrInfoMapper attrInfoMapper;

    @Autowired
    private ProjectInfoMapper projectInfoMapper;

    /**
     * 查询商品规格列表
     *
     * @param ruleInfo 商品规格
     * @return 商品规格
     */
    @Override
    public List<RuleInfo> list(RuleInfo ruleInfo) {
        LambdaQueryWrapper<RuleInfo> queryWrapper = new LambdaQueryWrapper<>();


        if (ObjUtils.notNull(ruleInfo.getName())){
            queryWrapper.like(RuleInfo::getName, ruleInfo.getName());
        }

        if (ObjUtils.notNull(ruleInfo.getStatus())){
            queryWrapper.eq(RuleInfo::getStatus, ruleInfo.getStatus());
        }

        if (ObjUtils.notNull(ruleInfo.getRemark())){
            queryWrapper.eq(RuleInfo::getRemark,ruleInfo.getRemark());
        }





        return list(queryWrapper);
    }

    /**
     * 添加规格信息
     *
     * @param ruleInfoAddModel 规格添加模型
     *
     * @return 时候成功
     */
    @Override
    public boolean save (RuleInfoAddModel ruleInfoAddModel) {
        // 验证状态值
        if (!Arrays.asList("Y", "N").contains(ruleInfoAddModel.getStatus())) {
            throw new RuntimeException("状态值无效，必须是 'Y' 或 'N'");
        }

        RuleInfo ruleInfo = RuleInfo.addModelBuild(ruleInfoAddModel, SecurityUtils::getUsername);
        boolean save = this.save(ruleInfo);

        // 保存规格属性
        if (!ruleInfoAddModel.getRuleAttrList().isEmpty()) {
            ruleAttrInfoService.saveBatch(
                    ruleInfoAddModel.getRuleAttrList().stream()
                            .map(ruleAttrAddModel -> RuleAttrInfo.addModelBuild(
                                    ruleAttrAddModel,
                                    ruleInfo::getId,
                                    SecurityUtils::getUsername
                            ))
                            .toList()
            );
        }

        return save;
    }

    /**
     * 查询规格信息
     * @param ruleInfoQueryReq 规格查询
     * @return 规格信息
     */
    @Override
    public TableDataInfo<RuleInfoResp> queryList (RuleInfoQueryReq ruleInfoQueryReq) {
        List<RuleInfo> list = this.list(RuleInfo.queryBuild(ruleInfoQueryReq));
        List<RuleInfoResp> ruleInfoRespList = list.stream()
                .map(ruleInfo -> RuleInfoResp.infoBuild(
                        ruleInfo,
                        ruleId -> {
                            LambdaQueryWrapper<RuleAttrInfo> queryWrapper = new LambdaQueryWrapper<>();
                            queryWrapper.eq(RuleAttrInfo::getRuleId, ruleId);
                            return ruleAttrInfoService.list(queryWrapper).stream()
                                    .map(RuleAttrAddModel::infoBuild)
                                    .toList();
                        },
                        this::checkRuleIsUsed  // 添加检查规格使用状态的方法
                ))
                .toList();
        boolean isPage = ruleInfoQueryReq.getParams().get("isPage") == null
                || Convert.toBool(ruleInfoQueryReq.getParams().get("isPage"), true);

        return TableDataInfo.<RuleInfoResp>builder()
                .rows(ruleInfoRespList)
                .total(isPage ? new PageInfo<>(list).getTotal() : 0)
                .build();
    }

    private Boolean checkRuleIsUsed(Long ruleId) {
        // 检查商品信息表中是否存在使用该规格的商品
        LambdaQueryWrapper<ProjectInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(ProjectInfo::getRuleId, ruleId);
        return projectInfoMapper.exists(queryWrapper);
    }

//    @Override
//    public RuleInfoAddModel getUpdById(Long id) {
//
//        RuleInfo ruleInfo = this.getById(id);
//        if (ruleInfo==null){
//            throw new RuntimeException("没有这个商品规格");
//        }
//
//        List<RuleAttrAddModel> ruleAttrInfos = ruleAttrInfoService.getRuleAttrInfo(ruleInfo.getId());
//        ArrayList<RuleAttrAddModel> arrayList = new ArrayList<>();
//
//        RuleAttrAddModel ruleAttrAddModel = new RuleAttrAddModel();
//
//        for (RuleAttrAddModel ruleAttrInfo : ruleAttrInfos) {
//            ruleAttrAddModel.setName(ruleAttrInfo.getName());
//        }
//
////        List<String> list = ruleAttrInfos.stream().map((RuleAttrAddModel t) -> RuleAttrInfo.getAttrValue(t)).toList();
//
//        ruleAttrAddModel.setValueList(list);
//        arrayList.add(ruleAttrAddModel);
//
//        RuleInfoAddModel resp = new RuleInfoAddModel();
//        resp.setId(ruleInfo.getId());
//        resp.setName(ruleInfo.getName());
//        resp.setStatus(ruleInfo.getStatus());
//        resp.setRuleAttrList(arrayList);
//
//        return resp;
//    }


    @Override
    public RuleInfoResp getUpdById(Long id){
//        RuleInfoResp getRuleById = attrInfoMapper.getRuleAttrInfo2(id);
//        List<RuleAttrAddModel> ruleAttrList = getRuleById.getRuleAttrList();
//
//        for (RuleAttrAddModel ruleAttrAddModel : ruleAttrList) {
//            String attrValue = ruleAttrAddModel.getAttrValue();
//            String[] split = attrValue.split(",");
//            ruleAttrAddModel.setValueList(List.of(split));
//        }
//        return getRuleById;
        RuleInfo ruleInfo = this.getById(id);
        if (ruleInfo == null) {
            throw new RuntimeException("商品规格不存在");
        }

        // 2. 查询规格属性信息
        LambdaQueryWrapper<RuleAttrInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(RuleAttrInfo::getRuleId, id);
        List<RuleAttrInfo> attrInfoList = ruleAttrInfoService.list(queryWrapper);

        // 3. 转换属性信息
        List<RuleAttrAddModel> ruleAttrList = attrInfoList.stream()
                .map(attrInfo -> {
                    RuleAttrAddModel model = new RuleAttrAddModel();
                    model.setName(attrInfo.getName());
                    // 处理属性值列表
                    if (StringUtils.isNotEmpty(attrInfo.getAttrValue())) {
                        List<String> valueList = Arrays.asList(attrInfo.getAttrValue().split(","));
                        // 去除空值和空白字符
                        valueList = valueList.stream()
                                .filter(StringUtils::isNotEmpty)
                                .map(String::trim)
                                .collect(Collectors.toList());
                        model.setValueList(valueList);
                    }
                    return model;
                })
                .collect(Collectors.toList());

        // 4. 构建返回对象
        return RuleInfoResp.builder()
                .id(ruleInfo.getId())
                .name(ruleInfo.getName())
                .status(ruleInfo.getStatus())
                .remark(ruleInfo.getRemark())
                .ruleAttrList(ruleAttrList)
                .createTime(ruleInfo.getCreateTime())
                .updateTime(ruleInfo.getUpdateTime())
                .build();
    }

    @Override
    public RuleInfoResp getUpdById2(Long id) {
        // 1. 查询基础规格信息
        RuleInfo ruleInfo = this.getById(id);
        if (ruleInfo == null) {
            throw new RuntimeException("商品规格不存在");
        }

        // 2. 查询规格属性信息并转换
        LambdaQueryWrapper<RuleAttrInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(RuleAttrInfo::getRuleId, id);
        List<RuleAttrAddModel> ruleAttrList = ruleAttrInfoService.list(queryWrapper).stream()
                .map(attrInfo -> {
                    RuleAttrAddModel model = new RuleAttrAddModel();
                    model.setName(attrInfo.getName());
                    model.setValueList(Arrays.asList(attrInfo.getAttrValue().split(",")));
                    return model;
                })
                .collect(Collectors.toList());

        // 3. 构建返回对象
        return RuleInfoResp.builder()
                .id(ruleInfo.getId())
                .name(ruleInfo.getName())
                .status(ruleInfo.getStatus())
                .ruleAttrList(ruleAttrList)
                .build();
    }

    @Override
    public RuleInfoResp getUpdById3(Long id) {
        RuleInfo ruleInfo = this.getById(id);
        if (ruleInfo==null){
            throw new RuntimeException("商品规格不存在");
        }
        LambdaQueryWrapper<RuleAttrInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(RuleAttrInfo::getRuleId,id);
        List<RuleAttrAddModel> ruleAttrList = ruleAttrInfoService.list(queryWrapper).stream()
                .map(attrInfo -> {
                    RuleAttrAddModel model = new RuleAttrAddModel();
                    model.setName(attrInfo.getName());
                    model.setValueList(Arrays.asList(attrInfo.getAttrValue().split(",")));
                    return model;
                })
                .collect(Collectors.toList());

        return RuleInfoResp.builder()
                .id(ruleInfo.getId())
                .name(ruleInfo.getName())
                .status(ruleInfo.getStatus())
                .ruleAttrList(ruleAttrList)
                .build();
    }


    @Override
    public void edit(Long id, RuleAttrInfoReq req) {
        attrInfoMapper.updRule(id,req);
        attrInfoMapper.deleteRule(id);

        List<RuleAttrAddModel> ruleAttrList = req.getRuleAttrList();
        for (RuleAttrAddModel ruleAttrAddModel : ruleAttrList) {
            List<String> valueList = ruleAttrAddModel.getValueList();
            String value="";
            String valueData=null;
            for (String s : valueList) {
                valueData = value + "," +s ;
            }
            String substring = valueData.substring(1);
            System.out.println(substring);
            attrInfoMapper.addRuleAttrModel(id,ruleAttrAddModel.getName(),substring);
        }


    }

    @Override
    public void edit2(Long id, RuleAttrInfoReq req) {
        RuleInfo ruleInfo = RuleInfo.builder()
                .id(id)
                .name(req.getName())
                .status(req.getStatus())
                .updateTime(new Date())
                .updateBy(SecurityUtils.getUsername())
                .build();
        this.updateById(ruleInfo);

        // 2. 删除原有的规格属性
        LambdaQueryWrapper<RuleAttrInfo> deleteWrapper = new LambdaQueryWrapper<>();
        deleteWrapper.eq(RuleAttrInfo::getRuleId, id);
        ruleAttrInfoService.remove(deleteWrapper);

        // 3. 批量新增规格属性
        if (ObjUtils.notNull(req.getRuleAttrList())) {
            List<RuleAttrInfo> attrInfoList = req.getRuleAttrList().stream()
                    .map(attr -> {
                        // 处理属性值列表转换为逗号分隔的字符串
                        String attrValue = attr.getValueList().stream()
                                .filter(StringUtils::isNotEmpty)
                                .map(String::trim)
                                .collect(Collectors.joining(","));

                        return RuleAttrInfo.builder()
                                .ruleId(id)
                                .name(attr.getName())
                                .attrValue(attrValue)
                                .createTime(new Date())
                                .updateTime(new Date())
                                .createBy(SecurityUtils.getUsername())
                                .updateBy(SecurityUtils.getUsername())
                                .build();
                    })
                    .collect(Collectors.toList());

            ruleAttrInfoService.saveBatch(attrInfoList);
        }
    }

    @Override
    public void edit3(Long id, RuleAttrInfoReq req) {
        // 更新规格基本信息
        updateById(RuleInfo.builder()
                .id(id)
                .name(req.getName())
                .remark(req.getRemark())
                .status(req.getStatus())
                .updateTime(new Date())
                .build());

        // 删除并新增规格属性
        ruleAttrInfoService.remove(new LambdaQueryWrapper<RuleAttrInfo>()
                .eq(RuleAttrInfo::getRuleId, id));

        if (ObjUtils.notNull(req.getRuleAttrList())) {
            ruleAttrInfoService.saveBatch(req.getRuleAttrList().stream()
                    .map(attr -> RuleAttrInfo.builder()
                            .ruleId(id)
                            .name(attr.getName())
                            .attrValue(String.join(",", attr.getValueList()))
                            .createTime(new Date())
                            .updateTime(new Date())
                            .createBy(SecurityUtils.getUsername())
                            .build())
                    .collect(Collectors.toList()));
        }
    }


}
